/*
 * Copyright (C) 2006-2007 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.codehaus.mojo.groovy.runtime.loader;

import org.apache.commons.lang.time.StopWatch;
import org.codehaus.mojo.groovy.feature.Provider;
import org.codehaus.mojo.groovy.feature.ProviderException;
import org.codehaus.mojo.groovy.feature.ProviderManager;
import org.codehaus.mojo.groovy.feature.ProviderRegistry;
import org.codehaus.mojo.groovy.feature.ProviderSelector;

/**
 * ???
 *
 * @plexus.component role="org.codehaus.mojo.groovy.feature.ProviderManager" instantiation-strategy="singleton"
 *
 * @version $Id: DefaultProviderManager.java 5426 2007-10-22 22:52:50Z user57 $
 * @author <a href="mailto:jason@planet57.com">Jason Dillon</a>
 */
public class DefaultProviderManager
    extends LoaderSupport
    implements ProviderManager
{
    /**
     * @plexus.requirement
     *
     * @noinspection UnusedDeclaration
     */
    private ProviderRegistry registry;

    /**
     * @plexus.requirement
     *
     * @noinspection UnusedDeclaration
     */
    private ProviderSelector selector;

    public ProviderRegistry getRegistry() {
        if (registry == null) {
            throw new IllegalStateException("Registry not bound");
        }

        return registry;
    }

    public ProviderSelector getSelector() {
        if (registry == null) {
            throw new IllegalStateException("Selector not bound");
        }

        return selector;
    }

    //
    // TODO: Expose more default selection API
    //

    private String getDefaultSelection() {
        return ProviderSelector.SELECT_DEFAULT;
    }

    //
    // TODO: Add loader and provider dynamic look up?
    //

    public Provider select(final String selection) {
        assert selection != null;

        log.debug("Selecting provider; selection: {}", selection);

        StopWatch watch = new StopWatch();
        watch.start();

        Provider provider;

        try {
            provider = getSelector().select(getRegistry(), selection);
        }
        catch (Exception e) {
            throw new ProviderException("Selection of provider failed; selection: " + selection, e);
        }

        if (provider == null) {
            throw new ProviderException("No providers found matching selection: " + selection);
        }

        watch.stop();

        log.debug("Selected provider: {} ({})", provider, watch);

        return provider;
    }

    public Provider select() {
        return select(getDefaultSelection());
    }
}